import {
  select,
  json,
  tree,
  hierarchy,
  linkHorizontal,
  zoom,
  event
} from 'd3';
import { treeData } from './data.js';

const svg = select('svg');
const height = svg.attr('height');
const width = svg.attr('width');

const margin = { top: -70, right: 130, bottom: 30, left: 120};
const innerWidth = width - margin.left - margin.right;
const innerHeight = height - margin.top - margin.bottom;

const zoomG = svg
    .attr('width', width)
    .attr('height', height)
  .append('g');

const g = svg.append('g')
    .attr('transform', `translate(${margin.left},${margin.top})`);

svg.call(zoom().on('zoom', () => {
  zoomG.attr('transform', event.transform);
}));

//Generated Treelayout with computed height and width 
const treeLayout = tree().size([innerHeight, innerWidth]);

//Declarating variables parameters
var i = 0,
    duration = 100,
    root;
root = hierarchy(treeData);

const generateTranslate = d => `translate(${d.y},${d.x})`;

const update = (source, collapseNode, deleteNode) => {  
  // console.log("New Update");
	const links = treeLayout(root).links();
  const linkPathGenerator = linkHorizontal()
    .x(d => d.y)
    .y(d => d.x);
  
  const linksData = g.selectAll('path').data(links);
  linksData
    .enter().append('path')    	
  	.merge(linksData)  
  		.attr('d', linkPathGenerator)
  		.transition().duration(duration-1)
  		.style('stroke-opacity', .5);
  
  linksData.exit()
  	.transition().duration(duration)
  		.style('stroke-opacity', 0)
    .remove();
  
  // Logic for Nodes
  const nodes = root.descendants(); 
 
  
  //For each node a group is created
  const nodesGroup = g.selectAll('g')
    .data(nodes,d=>d.data.id);
  
  //groupEnter is the entry stage
  const nodesGroupsEnter = nodesGroup.enter().append('g');  //Appending groups for each node
  
  nodesGroupsEnter
  	.attr('transform', (d, i) => generateTranslate(d) )  //Moving groups to correct location on initially  	
  	.on('click', d => collapseNode(d));
  
  nodesGroupsEnter //Enter Node Rectangle
    .append('rect')  	
  		.attr('x', d => - (d.data.name.length * 14) / 2)
  		.attr('y', -15)
  		.attr('width', d => d.data.name.length * 14 + 4)
    	.attr('height', 35)
    	.attr('rx', 4)  	
  		.attr('fill', d => d.data.color);
  
 	nodesGroupsEnter //Enter Node Label
    .append('text')      
      .attr('dy', '0.32em')
      .attr('text-anchor', 'middle')
    	.attr('font-size', d => 1.85 + 'em')
      .text(d => d.data.name);   
  
  //Update Stage
	const nodesGroupsUpdate = nodesGroupsEnter   			  		
    .merge(nodesGroup);
  
  nodesGroupsUpdate
  	.transition().duration(duration)
      .attr('transform', (d, i) => generateTranslate(d) ); //Moving groups to correct location on update
  
  // nodesGroupsUpdate.select('rect')
  // 	.transition().duration(duration)
  // 		.attr('stroke-width', d => d.data.id === hoveredNode? 4 : 2)
  // 		.attr('stroke', d => d.data.id === hoveredNode? 'black' : 'grey');
	
  // Remove any exiting nodes  
   
  var nodesGroupsExit = nodesGroup.exit().transition()
      .duration(duration)
      .remove();
	
  // On exit reduce the node circles size to 0
  nodesGroupsExit.select('rect')
  	.style('fill-opacity', 0)
    // .attr('width', 0)
    // .attr('height', 0);

  // On exit reduce the opacity of text labels
  nodesGroupsExit.select('text')
    .style('fill-opacity', 0);
  
}

const collapseNode = d => {  
  console.log('single click');
  if (d.children) {
    d._children = d.children;
    d.children = null;
  } 
  else {      
    d.children = d._children;
    d._children = null;
  }   
  update(d,  collapseNode, deleteNode);  
}

const deleteNode = d => {
  
  if(d.children) {
    if(d._children) {
    d._children = d.children;
    d.children = null; 
  }
}
update(root, collapseNode, deleteNode);


